@@ -89,7 +89,7 @@ void help_about_callback ()
 	  NULL
   };
 
-  const gchar *translator_credits = _("");
+  const gchar *translator_credits = _("translator_credits");
 
 #ifdef ENABLE_NLS
   {
@@ -109,7 +109,7 @@ void help_about_callback ()
 			    "solitaire card engine that allows many different games to be played"),
 			    (const char **)authors,
 			    (const char **)documenters,
-			    (const char *)translator_credits,
+			    strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 			    NULL);
   gtk_signal_connect (GTK_OBJECT (about),
 		      "destroy",
@@ -235,7 +235,7 @@ void create_menus ()
 
   /* Kids, don't try this at home */
   menu_item = game_menu [1].widget;
-  menu = scroll_menu_new ();
+  menu = gtk_menu_new ();
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
   w = gtk_tearoff_menu_item_new ();
   gtk_widget_show(w);
