@@ -28,11 +28,12 @@
 
 #include <sys/time.h>
 #include <string.h>
+#include <games-clock.h>
+#include <gconf/gconf-client.h>
 
 #include "gataxx.h"
 #include "ataxx.h"
 #include "properties.h"
-#include "clock.h"
 
 GdkPixmap *buffer_pixmap = NULL;
 GdkPixmap *tiles_pixmap = NULL;
@@ -46,6 +47,8 @@ GtkWidget *black_score;
 GtkWidget *white_score;
 GtkWidget *time_display;
 
+GConfClient *gataxx_gconf_client = NULL;
+
 gint flip_pixmaps_id = 0;
 gint statusbar_id;
 guint whose_turn = BLACK_TURN;
@@ -94,8 +97,7 @@ int session_xpos = -1;
 int session_ypos = -1;
 int session_position = 0;
 
-gchar tile_set[255];
-gchar tile_set_tmp[255];
+gchar *tile_set = NULL;
 
 guint selected_x;
 guint selected_y;
@@ -337,9 +339,9 @@ void undo_move_cb(GtkWidget *widget, gpointer data) {
   gui_status();
   
   if(timer_valid) {
-    clock_stop(CLOCK(time_display));
+    games_clock_stop(GAMES_CLOCK(time_display));
     gtk_widget_set_sensitive(time_display, FALSE);
-    clock_set_seconds(CLOCK(time_display), 0);
+    games_clock_set_seconds(GAMES_CLOCK(time_display), 0);
     timer_valid = 0;
   }
   
@@ -353,15 +355,15 @@ void black_level_cb(GtkWidget *widget, gpointer data) {
   
   tmp = atoi((gchar *)data);
   
-  gnome_config_set_int("/gataxx/Preferences/blacklevel", tmp);
-  gnome_config_sync();
+  gconf_client_set_int(gataxx_gconf_client, "/apps/gataxx/blacklevel",
+                       tmp, NULL);
   
   black_computer_level = tmp;
   
   if(game_in_progress) {
-    clock_stop(CLOCK(time_display));
+    games_clock_stop(GAMES_CLOCK(time_display));
     gtk_widget_set_sensitive(time_display, FALSE);
-    clock_set_seconds(CLOCK(time_display), 0);
+    games_clock_set_seconds(GAMES_CLOCK(time_display), 0);
     timer_valid = 0;
   }
   
@@ -374,15 +376,15 @@ void white_level_cb(GtkWidget *widget, gpointer data)
   
   tmp = atoi((gchar *)data);
   
-  gnome_config_set_int("/gataxx/Preferences/whitelevel", tmp);
-  gnome_config_sync();
+  gconf_client_set_int(gataxx_gconf_client, "/apps/gataxx/whitelevel",
+                       tmp, NULL);
   
   white_computer_level = tmp;
   
   if(game_in_progress) {
-    clock_stop(CLOCK(time_display));
+    games_clock_stop(GAMES_CLOCK(time_display));
     gtk_widget_set_sensitive(time_display, FALSE);
-    clock_set_seconds(CLOCK(time_display), 0);
+    games_clock_set_seconds(GAMES_CLOCK(time_display), 0);
     timer_valid = 0;
   }
   
@@ -392,26 +394,39 @@ void white_level_cb(GtkWidget *widget, gpointer data)
 void about_cb(GtkWidget *widget, gpointer data) {
 
   static GtkWidget *about;
-  
+  GdkPixbuf *pixbuf = NULL;
   const gchar *authors[] = {"Chris Rogers", NULL};
   gchar *documenters[] = {
                           NULL
                           };
   /* Translator credits */
-  gchar *translator_credits = _("");
+  gchar *translator_credits = _("translator_credits");
   
   if (about != NULL) {
     gdk_window_raise (about->window);
     gdk_window_show (about->window);
     return;
   }
+
+  {
+	  char *filename = NULL;
+
+	  filename = gnome_program_locate_file (NULL,
+			  GNOME_FILE_DOMAIN_PIXMAP,  ("gataxx.png"),
+			  TRUE, NULL);
+	  if (filename != NULL)
+	  {
+		  pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
+		  g_free (filename);
+	  }
+  }
   
   about = gnome_about_new(_("gataxx"), VERSION, "(C) 1999 Chris Rogers",
   			  _("gataxx is a Gnome port of the old game ataxx.  It is derived from Ian Peters' iagno code."),
   			  (const char **)authors,
   			  (const char **)documenters,
-                          (const char *)translator_credits,
-			  NULL);
+  			  strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			  pixbuf);
   g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK
 		      (gtk_widget_destroyed), &about);
   gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW(window));
@@ -546,7 +561,7 @@ void load_pixmaps() {
     g_print(_("Could not find \'%s\' pixmap file for gataxx\n"), fname);
     exit(1);
   }
-  
+
   image = gdk_pixbuf_new_from_file(fname, NULL);
   gdk_pixbuf_render_pixmap_and_mask (image, &tiles_pixmap, &tiles_mask, 127);
   
@@ -628,7 +643,7 @@ gint flip_pixmaps(gpointer data) {
 void init_new_game() {
   
   guint i, j;
-  
+
   if (flip_final_id) {
     gtk_timeout_remove(flip_final_id);
     flip_final_id = 0;
@@ -667,12 +682,12 @@ void init_new_game() {
   whose_turn = BLACK_TURN;
   gui_message(_("Dark's move"));
   
-  clock_stop(CLOCK(time_display));
-  clock_set_seconds(CLOCK(time_display), 0);
+  games_clock_stop(GAMES_CLOCK(time_display));
+  games_clock_set_seconds(GAMES_CLOCK(time_display), 0);
   
   if(black_computer_level ^ white_computer_level) {
     if(!black_computer_level)
-      clock_start(CLOCK(time_display));
+      games_clock_start(GAMES_CLOCK(time_display));
     gtk_widget_set_sensitive(time_display, TRUE);
     timer_valid = 1;
   } else {
@@ -743,7 +758,7 @@ void create_window() {
   
   gtk_table_attach(GTK_TABLE(table), sep, 6, 7, 0, 1, 0, GTK_FILL, 3, 3);
 
-  time_display = clock_new();
+  time_display = games_clock_new();
   gtk_widget_set_sensitive(time_display, FALSE);
   gtk_widget_show(time_display);
   
@@ -814,46 +829,45 @@ void set_bg_color() {
   gdk_image_unref(tmpimage);
 }
 
-static char *nstr(int n) {
-
-  char buf[20];
-  sprintf(buf, "%d", n);
-  return strdup(buf);
-}
-
 static int save_state(GnomeClient *client, gint phase, GnomeRestartStyle save_style, gint shutdown, GnomeInteractStyle interact_style, gint fast, gpointer client_data) {
 
   char *argv[20];
   int i;
-  gint xpos, ypos;
+  int xpos, ypos;
   
   gdk_window_get_origin(window->window, &xpos, &ypos);
   
   i = 0;
   argv[i++] = (char *)client_data;
   argv[i++] = "-x";
-  argv[i++] = nstr(xpos);
+  argv[i++] = g_strdup_printf("%d",xpos);
   argv[i++] = "-y";
-  argv[i++] = nstr(ypos);
+  argv[i++] = g_strdup_printf("%d",ypos);
   
   gnome_client_set_restart_command(client, i, argv);
   gnome_client_set_clone_command(client, 0, NULL);
   
-  free(argv[2]);
-  free(argv[4]);
+  g_free(argv[2]);
+  g_free(argv[4]);
   
   return TRUE;
 }
 
-int main(int argc, char **argv) {
+void
+key_changed_cb (GConfClient *tmp_client, guint cnx_id, GConfEntry *tmp_entry,
+                gpointer tmp_data)
+{
+	reload_properties ();
+}
 
-  GnomeClient *client;
+int main(int argc, char **argv) {
+    GnomeClient *client;
 
   /*  CORBA_def(CORBA_Environment ev;) */
     struct timeval tv;
     
     gnome_score_init("gataxx");
-
+  
     bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain(GETTEXT_PACKAGE);
@@ -873,10 +887,17 @@ int main(int argc, char **argv) {
     
     g_signal_connect(G_OBJECT(client), "save_yourself", G_CALLBACK(save_state), argv[0]);
     g_signal_connect(G_OBJECT(client), "die", G_CALLBACK(quit_game_cb), argv[0]);
-  
+
+    gconf_init(argc, argv, NULL);
+    gataxx_gconf_client = gconf_client_get_default();
+    gconf_client_add_dir(gataxx_gconf_client, "/apps/gataxx",
+                         GCONF_CLIENT_PRELOAD_NONE, NULL);
+    gconf_client_notify_add(gataxx_gconf_client, "/apps/gataxx",
+                            key_changed_cb, NULL, NULL, NULL);
+
     create_window();
-    
-    load_properties ();
+
+    load_properties();
     
     load_pixmaps();
     
@@ -894,5 +915,7 @@ int main(int argc, char **argv) {
     
     gtk_main();
     
+    g_object_unref(G_OBJECT(gataxx_gconf_client));
+
     return 0;
 }
